@@ -1,7 +1,11 @@
-setMethod("rrecsys", signature = c("dataSet"), function(data, alg, ...) {
+setMethod("rrecsys", 
+          signature = c("_ds"), 
+          function(data, alg, ...) {
+            
     recom <- rrecsysRegistry$get_entry(alg = alg)
     if (is.null(recom)) 
         stop(paste("Wrong method!!!"))
     
     recom$fun(data, ...)
+    
 }) 
